/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_CHECKSUM_H
#define __TOOLBOX_CHECKSUM_H


namespace toolbox
{
	/**
	 * The ChecksumCalculator class provides functionality
	 * for checksum generation, such as CRC32.
	 * @author Thomas Jacob
	 */
	class ChecksumCalculator
	{
		private:

			/**
			 * The CRC32 calculation table.
			 */
			unsigned long Crc32Table[256];

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new checksum calculator.
			 */
			ChecksumCalculator();

			/**
			 * Destroys the checksum calculator.
			 */
			~ChecksumCalculator();

			/**
			 * Calculates the CRC32 from a given buffer.
			 * @param buffer The buffer to calculate the checksum on.
			 * @param len The length of the buffer in bytes.
			 * @return The CRC32 of the buffer.
			 */
			unsigned long GetCrc32(const char * buffer, int len);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};
}


#endif
