/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class Config ---

Config::Config()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	if ((Content = new ConfigSection("Root")) == NULL)
		throw EOutOfMemory::Get();
}

Config::~Config()
{
	ASSERTION_THIS;

	if (Content) {delete Content; Content = NULL;}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

void Config::Clear()
{
	ASSERTION_THIS;

	GetContent()->DeleteAllParameters();
	GetContent()->DeleteAllSections();
}

bool Config::ExportIni()
{
	ASSERTION_THIS;

	Path * IniPath = Path::CreateApplicationPath();
	PathElement * IniPathTail = IniPath->GetLastElement();
	if (IniPathTail == NULL)
	{
		delete IniPath;
		return false;
	}
	IniPathTail->ChangeExtension("ini");

	char * iniFileName = IniPath->ToString(false);
	delete IniPath;

	bool result = ExportIni(iniFileName);
	delete iniFileName;

	return result;
}

bool Config::ExportIni(const char * fileName)
{
	ASSERTION_THIS;

	FILE * file;
	if ((file = fopen(fileName, "wt")) == NULL)
		return false;

	PointeredList * ParameterNames = Content->GetParameterNames();
	ParameterNames->Sort(PointeredList::CaseInsensitiveStringCompare);
	PointeredList * SectionNames = Content->GetSectionNames();
	SectionNames->Sort(PointeredList::CaseInsensitiveStringCompare);

	PointeredListItem * ParameterName = ParameterNames->GetFirst();
	while (ParameterName)
	{
		ConfigParameter * Parameter = Content->GetParameter((char *) ParameterName->GetData());
		fprintf(file, "%s=%s\n", (char *) ParameterName->GetData(), Parameter->GetStringValue());

		ParameterName = ParameterName->GetNext();
	}
	if (ParameterNames->GetFirst())
		fprintf(file, "\n");

	PointeredListItem * SectionName = SectionNames->GetFirst();
	while (SectionName)
	{
		fprintf(file, "[%s]\n", (char *) SectionName->GetData());

		ConfigSection * Section = Content->GetSection((char *) SectionName->GetData());
		PointeredList * ParameterNames = Section->GetParameterNames();
		ParameterNames->Sort(PointeredList::CaseInsensitiveStringCompare);
		PointeredListItem * ParameterName = ParameterNames->GetFirst();
		while (ParameterName)
		{
			ConfigParameter * Parameter = Section->GetParameter((char *) ParameterName->GetData());
			fprintf(file, "%s=%s\n", (char *) ParameterName->GetData(), Parameter->GetStringValue());

			ParameterName = ParameterName->GetNext();
		}
		delete ParameterNames;
		fprintf(file, "\n");

		SectionName = SectionName->GetNext();
	}

	delete ParameterNames;
	delete SectionNames;

	fclose(file);
	return true;
}

bool Config::ImportIni()
{
	ASSERTION_THIS;

	Path * IniPath = Path::CreateApplicationPath();
	PathElement * IniPathTail = IniPath->GetLastElement();
	if (IniPathTail == NULL)
	{
		delete IniPath;
		return false;
	}
	IniPathTail->ChangeExtension("ini");

	char * iniFileName = IniPath->ToString(false);
	delete IniPath;

	bool result = ImportIni(iniFileName);
	delete iniFileName;

	return result;
}

bool Config::ImportIni(const char * fileName)
{
	ASSERTION_THIS;

	Clear();

	FILE * file;
	if ((file = fopen(fileName, "rt")) == NULL)
		return false;

	if (Content) {delete Content; Content = NULL;}
	if ((Content = new ConfigSection("Root")) == NULL)
	{
		fclose(file);
		throw EOutOfMemory::Get();
	}

	ConfigSection * CurrentSection = Content;
	char line[512];

	fgets(line, 512, file);
	while (!feof(file))
	{
		while (line[0] && strchr(" \r\n\t\f", line[0]))
			memmove(line, &line[1], strlen(&line[1])+1);
		while (line[0] && strchr(" \r\n\t\f", line[strlen(line)-1]))
			line[strlen(line)-1] = 0;

		if (line[0] == '#' || line[0] == ';')
		{
			fgets(line, 512, file);
			continue;
		}

		char * equalsPtr;
		if ((equalsPtr = strchr(line, '=')) != NULL)
		{
			equalsPtr[0] = 0;
			char * key = line;
			char * value = &equalsPtr[1];

			if (!key[0] || !value[0])
			{
				fgets(line, 512, file);
				continue;
			}

			CurrentSection->GetParameter(key)->SetStringValue(value);
		}
		else
			if (line[0] == '[' && line[strlen(line)-1] == ']')
			{
				line[strlen(line)-1] = 0;
				CurrentSection = Content->GetSection(&line[1]);
			}

		fgets(line, 512, file);
	}

	fclose(file);

	return true;
}

#ifdef _TOOLBOX_TEST
	int Config::InstanceCount = 0;
#endif

bool Config::Parse(char * text)
{
	ASSERTION_THIS;

	Clear();

	PointeredList * Tokens;
	if ((Tokens = new PointeredList) == NULL)
		throw EOutOfMemory::Get();

	PointeredListItem * Token;
	if ((Token = new PointeredListItem) == NULL)
		throw EOutOfMemory::Get();
	Token->SetData(GetContent());
	Tokens->Append(Token);

	bool error = false;
	bool stringWasSet = false;
	char * restPtr = text;
	char * endPtr = NULL;
	while (!error && restPtr[0])
	{
		// Find the end of the next token
		endPtr = restPtr;
		if (endPtr[0] == '<')
			endPtr++;
		while (endPtr[0] && !error)
		{
			char * backSlashPtr = strchr(endPtr, '\\');
			char * openingBracketPtr = strchr(endPtr, '<');
			char * closingBracketPtr = strchr(endPtr, '>');

			if (backSlashPtr != NULL && (openingBracketPtr == NULL || backSlashPtr < openingBracketPtr)
				&& (closingBracketPtr == NULL || backSlashPtr < closingBracketPtr))
			{
				endPtr = &backSlashPtr[1];
				if (endPtr[0]) endPtr++;
			}
			else
			{
				if (openingBracketPtr != NULL
					&& (closingBracketPtr == NULL || openingBracketPtr < closingBracketPtr))
					endPtr = openingBracketPtr;
				else if (closingBracketPtr != NULL
					&& (openingBracketPtr == NULL || closingBracketPtr < openingBracketPtr))
					endPtr = &closingBracketPtr[1];
				else
					endPtr = &endPtr[strlen(endPtr)];

				break;
			}
		}

		// Copy and clean up token
		char merk = endPtr[0];
		endPtr[0] = 0;
		char * tokenString;
		if ((tokenString = new char[strlen(restPtr)+1]) == NULL)
			throw EOutOfMemory::Get();
		strcpy(tokenString, restPtr);
		while (tokenString[0] && strchr(" \r\n\t\f", tokenString[0]))
			memmove(tokenString, &tokenString[1], strlen(&tokenString[1])+1);
		while (tokenString[0] && strchr(" \r\n\t\f", tokenString[strlen(tokenString)-1]))
			tokenString[strlen(tokenString)-1] = 0;

		// Determine the token type and extract the pure token text
		int tokenType;
		if (tokenString[0] == '<' && tokenString[1] == '/' && tokenString[strlen(tokenString)-1] == '>')
		{
			tokenType = _CF_TOKENTYPE_CLOSING;
			tokenString[strlen(tokenString)-1] = 0;
			memmove(tokenString, &tokenString[2], strlen(&tokenString[2])+1);
		}
		else if (tokenString[0] == '<' && tokenString[strlen(tokenString)-1] == '>')
		{
			tokenType = _CF_TOKENTYPE_OPENING;
			tokenString[strlen(tokenString)-1] = 0;
			memmove(tokenString, &tokenString[1], strlen(&tokenString[1])+1);
		}
		else if (tokenString[0])
			tokenType = _CF_TOKENTYPE_PLAIN;
		else
			tokenType = _CF_TOKENTYPE_IGNORE;

		// Convert special characters in token text
		for (unsigned int i=0; i<strlen(tokenString);)
			if (tokenString[i] == '\\' && i < strlen(tokenString)-1)
			{
				memmove(&tokenString[i], &tokenString[i+1], strlen(&tokenString[i+1])+1);
				switch (tokenString[i])
				{
					case 'r': tokenString[i++] = '\r'; break;
					case 'n': tokenString[i++] = '\n'; break;
					case 't': tokenString[i++] = '\t'; break;
					case 'f': tokenString[i++] = '\f'; break;

					case '0':
						memmove(&tokenString[i], &tokenString[i+1], strlen(&tokenString[i+1])+1);
					break;

					default: i++; break;
				}
			}
			else
				i++;

		switch (tokenType)
		{
			case _CF_TOKENTYPE_OPENING:
			{
				PointeredListItem * Token;
				if ((Token = new PointeredListItem) == NULL)
					throw EOutOfMemory::Get();

				ConfigSection * LastSection = (ConfigSection *) Tokens->GetLast()->GetData();
				Token->SetData(LastSection->GetSection(tokenString));
				Tokens->Append(Token);
			}
			break;

			case _CF_TOKENTYPE_CLOSING:
			{
				ConfigSection * LastSection = (ConfigSection *) Tokens->GetLast()->GetData();
				if (Tokens->GetCount() <= 1 || (strcmp(tokenString, "") != 0
					&& strcmp(LastSection->GetName(), tokenString) != 0))
				{
					error = true;
					break;
				}

				if (stringWasSet)
				{
					ConfigSection * LastSection = (ConfigSection *) Tokens->GetLast()->GetData();
					ConfigSection * SecondButLastSection = (ConfigSection *) Tokens->GetLast()->GetPrev()->GetData();
					SecondButLastSection->DeleteSection(LastSection->GetName());
					stringWasSet = false;
				}
				Tokens->DeleteLast();
			}
			break;

			case _CF_TOKENTYPE_PLAIN:
			{
				if (Tokens->GetCount() < 2)
				{
					error = true;
					break;
				}

				ConfigSection * LastSection = (ConfigSection *) Tokens->GetLast()->GetData();
				ConfigSection * SecondButLastSection = (ConfigSection *) Tokens->GetLast()->GetPrev()->GetData();
				SecondButLastSection->GetParameter(LastSection->GetName())->SetStringValue(tokenString);
				stringWasSet = true;
			}
			break;
		}

		delete tokenString;

		endPtr[0] = merk;
		restPtr = endPtr;
	}

	if (Tokens->GetCount() != 1)
		error = true;

	Tokens->DeleteAll();
	delete Tokens;

	if (error)
		Clear();

	return !error;
}

bool Config::Read()
{
	ASSERTION_THIS;

	Path * CfgPath = Path::CreateApplicationPath();
	PathElement * CfgPathTail = CfgPath->GetLastElement();
	if (CfgPathTail == NULL)
	{
		delete CfgPath;
		return false;
	}
	CfgPathTail->ChangeExtension("cfg");

	char * cfgFileName = CfgPath->ToString(false);
	delete CfgPath;

	bool result = Read(cfgFileName);
	delete cfgFileName;

	return result;
}

bool Config::Read(const char * fileName)
{
	ASSERTION_THIS;

	FILE * file;
	if ((file = fopen(fileName, "rb")) == NULL)
		return false;

	fseek(file, 0L, SEEK_END);
	int len = ftell(file);
	fseek(file, 0L, SEEK_SET);

	char * buffer;
	if ((buffer = new char[len+1]) == NULL)
	{
		fclose(file);
		throw EOutOfMemory::Get();
	}

	fread(buffer, len, 1, file);
	buffer[len] = 0;
	fclose(file);

	bool success = Parse(buffer);
	delete buffer;
	return success;
}

#ifdef _TOOLBOX_TEST
	void Config::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif

bool Config::Write(bool withBacking, bool indent)
{
	ASSERTION_THIS;

	Path * CfgPath = Path::CreateApplicationPath();
	PathElement * CfgPathTail = CfgPath->GetLastElement();
	if (CfgPathTail == NULL)
	{
		delete CfgPath;
		return false;
	}
	CfgPathTail->ChangeExtension("cfg");

	char * cfgFileName = CfgPath->ToString(false);
	delete CfgPath;

	bool result = Write(cfgFileName, withBacking, indent);
	delete cfgFileName;

	return result;
}

bool Config::Write(const char * fileName, bool withBacking, bool indent)
{
	ASSERTION_THIS;

	bool success;
	if (withBacking)
	{
		Path * TempPath;
		if ((TempPath = new Path(fileName)) == NULL)
			throw EOutOfMemory::Get();

		PathElement * TempPathTail = TempPath->GetLastElement();
		if (TempPathTail == NULL)
		{
			delete TempPath;
			return false;
		}
		TempPathTail->ChangeExtension("tmp");

		char * tempFileName = TempPath->ToString(false);
		delete TempPath;

		Config::FileWriter * Writer;
		if ((Writer = new Config::FileWriter) == NULL)
			throw EOutOfMemory::Get();

		if (!Writer->Open(tempFileName))
		{
			delete Writer;
			delete tempFileName;
			return false;
		}
		success = Content->WriteTo(Writer, 0, indent);
		delete Writer;

		unlink(fileName);
		rename(tempFileName, fileName);
		delete tempFileName;
	}
	else
	{
		Config::FileWriter * Writer;
		if ((Writer = new Config::FileWriter) == NULL)
			throw EOutOfMemory::Get();

		if (!Writer->Open(fileName))
		{
			delete Writer;
			return false;
		}
		success = Content->WriteTo(Writer, 0, indent);
		delete Writer;
	}

	return success;
}

bool Config::Write(class Config::Writer * Writer, bool indent)
{
	ASSERTION_THIS;
	return Content->WriteTo(Writer, 0, indent);
}

bool Config::WriteCorrectedForCfgFile(class Config::Writer * Writer, const char * str)
{
	bool success = true;

	if (!str[0])
		success = success && Writer->Write("\\0");
	else
		for (unsigned int i=0; i<strlen(str); i++)
			switch (str[i])
			{
				case '\\':
				case '<':
				case '>':
				case '/':
					success = success && Writer->Write('\\');
					success = success && Writer->Write(str[i]);
				break;

				case '\r':
					success = success && Writer->Write("\\r");
				break;

				case '\n':
					success = success && Writer->Write("\\n");
				break;

				case '\t':
					success = success && Writer->Write("\\t");
				break;

				case '\f':
					success = success && Writer->Write("\\f");
				break;

				default:
					success = success && Writer->Write(str[i]);
				break;
			}

	return success;
}


// --- Class Config::FileWriter ---

Config::FileWriter::FileWriter()
{
	File = NULL;
}

Config::FileWriter::~FileWriter()
{
	ASSERTION_THIS;
	if (File != NULL) {fclose(File); File = NULL;}
}

bool Config::FileWriter::Open(const char * fileName)
{
	ASSERTION_THIS;
	ASSERTION(File == NULL);

	return (File = fopen(fileName, "wt")) != NULL;
}

bool Config::FileWriter::Write(char character)
{
	ASSERTION_THIS;
	return fputc(character, File) != EOF;
}

bool Config::FileWriter::Write(const char * string)
{
	ASSERTION_THIS;
	return fputs(string, File) >= 0;
}


// --- Class Config::StringWriter ---

Config::StringWriter::StringWriter()
{
	if ((Buffer = new char[CF_SWBUFFERINCREASE]) == NULL)
		throw EOutOfMemory::Get();
	Buffer[0] = 0;
	BufferLen = 1;
	BufferSize = CF_SWBUFFERINCREASE;
}

Config::StringWriter::~StringWriter()
{
	ASSERTION_THIS;
	if (Buffer != NULL) {delete Buffer; Buffer = NULL;}
}

void Config::StringWriter::IncreaseBuffer()
{
	char * newBuffer;
	if ((newBuffer = new char[BufferSize + CF_SWBUFFERINCREASE]) == NULL)
		throw EOutOfMemory::Get();

	memcpy(newBuffer, Buffer, BufferSize);
	delete Buffer;
	Buffer = newBuffer;

	BufferSize += CF_SWBUFFERINCREASE;
}

bool Config::StringWriter::Write(char character)
{
	ASSERTION_THIS;

	if (BufferLen >= BufferSize)
		IncreaseBuffer();

	Buffer[BufferLen - 1] = character;
	Buffer[BufferLen++] = 0;

	return true;
}

bool Config::StringWriter::Write(const char * string)
{
	ASSERTION_THIS;

	unsigned int stringLen = strlen(string);

	while (BufferLen + stringLen > BufferSize)
		IncreaseBuffer();

	strcat(Buffer, string);
	BufferLen += stringLen;

	return true;
}


// --- Class Config::Writer ---

Config::Writer::~Writer()
{
	ASSERTION_THIS;
}


// --- Class ConfigSection ---

ConfigSection::ConfigSection(const char * name)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Name = NULL;
	Parameters = NULL;
	Sections = NULL;

	if ((Name = new char[strlen(name)+1]) == NULL)
		throw EOutOfMemory::Get();
	strcpy(Name, name);

	if ((Parameters = new StringKeyHashMap<ConfigParameter>) == NULL)
	{
		delete Name; Name = NULL;
		throw EOutOfMemory::Get();
	}

	if ((Sections = new StringKeyHashMap<ConfigSection>) == NULL)
	{
		delete Name; Name = NULL;
		delete Parameters; Parameters = NULL;
		throw EOutOfMemory::Get();
	}
}

ConfigSection::~ConfigSection()
{
	ASSERTION_THIS;

	if (Name != NULL) {delete Name; Name = NULL;}

	if (Parameters)
	{
		Parameters->DeleteAll();
		delete Parameters;
		Parameters = NULL;
	}

	if (Sections)
	{
		Sections->DeleteAll();
		delete Sections;
		Sections = NULL;
	}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

bool ConfigSection::ContainsParameter(const char * name)
{
	ASSERTION_THIS;
	return Parameters->ContainsKey(name);
}

bool ConfigSection::ContainsSection(const char * name)
{
	ASSERTION_THIS;
	return Sections->ContainsKey(name);
}

void ConfigSection::DeleteAllParameters()
{
	ASSERTION_THIS;
	Parameters->DeleteAll();
}

void ConfigSection::DeleteAllSections()
{
	ASSERTION_THIS;
	Sections->DeleteAll();
}

void ConfigSection::DeleteParameter(const char * name)
{
	ASSERTION_THIS;
	Parameters->Delete(name);
}

void ConfigSection::DeleteSection(const char * name)
{
	ASSERTION_THIS;
	Sections->Delete(name);
}

bool ConfigSection::GetBoolValue(const char * name, bool defaultBoolValue)
{
	ASSERTION_THIS;

	ConfigParameter * Parameter = Parameters->Get(name);
	if (Parameter == NULL)
	{
		if ((Parameter = new ConfigParameter(defaultBoolValue)) == NULL)
			throw EOutOfMemory::Get();
		Parameters->Set(name, Parameter);
	}

	return Parameter->GetBoolValue();
}

double ConfigSection::GetDoubleValue(const char * name, double defaultDoubleValue)
{
	ASSERTION_THIS;

	ConfigParameter * Parameter = Parameters->Get(name);
	if (Parameter == NULL)
	{
		if ((Parameter = new ConfigParameter(defaultDoubleValue)) == NULL)
			throw EOutOfMemory::Get();
		Parameters->Set(name, Parameter);
	}

	return Parameter->GetDoubleValue();
}

int ConfigSection::GetIntValue(const char * name, int defaultIntValue)
{
	ASSERTION_THIS;

	ConfigParameter * Parameter = Parameters->Get(name);
	if (Parameter == NULL)
	{
		if ((Parameter = new ConfigParameter((long) defaultIntValue)) == NULL)
			throw EOutOfMemory::Get();
		Parameters->Set(name, Parameter);
	}

	return (int) Parameter->GetLongValue();
}

long ConfigSection::GetLongValue(const char * name, long defaultLongValue)
{
	ASSERTION_THIS;

	ConfigParameter * Parameter = Parameters->Get(name);
	if (Parameter == NULL)
	{
		if ((Parameter = new ConfigParameter(defaultLongValue)) == NULL)
			throw EOutOfMemory::Get();
		Parameters->Set(name, Parameter);
	}

	return Parameter->GetLongValue();
}

ConfigParameter * ConfigSection::GetParameter(const char * name)
{
	ASSERTION_THIS;

	ConfigParameter * Parameter = Parameters->Get(name);
	if (Parameter == NULL)
	{
		if ((Parameter = new ConfigParameter) == NULL)
			throw EOutOfMemory::Get();
		Parameters->Set(name, Parameter);
	}

	return Parameter;
}

ConfigSection * ConfigSection::GetSection(const char * name)
{
	ASSERTION_THIS;

	ConfigSection * Section = Sections->Get(name);
	if (Section == NULL)
	{
		if ((Section = new ConfigSection(name)) == NULL)
			throw EOutOfMemory::Get();
		Sections->Set(name, Section);
	}

	return Section;
}

const char * ConfigSection::GetStringValue(const char * name, const char * defaultStringValue)
{
	ASSERTION_THIS;

	ConfigParameter * Parameter = Parameters->Get(name);
	if (Parameter == NULL)
	{
		if ((Parameter = new ConfigParameter(defaultStringValue)) == NULL)
			throw EOutOfMemory::Get();
		Parameters->Set(name, Parameter);
	}

	return Parameter->GetStringValue();
}

bool ConfigSection::HasParameter(const char * name)
{
	ASSERTION_THIS;

	return Parameters->Get(name) != NULL;
}

bool ConfigSection::HasSection(const char * name)
{
	ASSERTION_THIS;

	return Sections->Get(name) != NULL;
}

#ifdef _TOOLBOX_TEST
	int ConfigSection::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void ConfigSection::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif

bool ConfigSection::WriteIndentation(class Config::Writer * Writer, int depth)
{
	bool success = true;
	for (int i=0; i<depth; i++)
		success = success && Writer->Write("\t");
	return success;
}

bool ConfigSection::WriteTo(class Config::Writer * Writer, int depth, bool indent)
{
	ASSERTION_THIS;

	bool success = true;

	PointeredList * SectionNames = GetSectionNames();
	SectionNames->Sort(PointeredList::CaseInsensitiveStringCompare);
	PointeredList * ParameterNames = GetParameterNames();
	ParameterNames->Sort(PointeredList::CaseInsensitiveStringCompare);

	PointeredListItem * ParameterName = ParameterNames->GetFirst();
	while (success && ParameterName)
	{
		ConfigParameter * Parameter = GetParameter((char *) ParameterName->GetData());

		if (indent)
			success = success && WriteIndentation(Writer, depth);

		success = success && Writer->Write("<");
		success = success && Config::WriteCorrectedForCfgFile(Writer, (char *) ParameterName->GetData());
		success = success && Writer->Write(">");
		success = success && Parameter->WriteCorrectedForCfgFile(Writer);
		success = success && Writer->Write("</>\n");

		ParameterName = ParameterName->GetNext();
	}

	if (indent && ParameterNames->GetFirst() && SectionNames->GetFirst())
		success = success && Writer->Write("\n");

	bool first = true;
	PointeredListItem * SectionName = SectionNames->GetFirst();
	while (success && SectionName)
	{
		if (indent)
			if (first)
				first = false;
			else
				success = success && Writer->Write("\n");

		if (indent)
			success = success && WriteIndentation(Writer, depth);
		success = success && Writer->Write("<");
		success = success && Config::WriteCorrectedForCfgFile(Writer, (char *) SectionName->GetData());
		success = success && Writer->Write(">\n");

		success = success && GetSection((char *) SectionName->GetData())->WriteTo(Writer, depth+1, indent);

		if (indent)
			success = success && WriteIndentation(Writer, depth);
		success = success && Writer->Write("</");
		success = success && Config::WriteCorrectedForCfgFile(Writer, (char *) SectionName->GetData());
		success = success && Writer->Write(">\n");

		SectionName = SectionName->GetNext();
	}

	delete SectionNames;
	delete ParameterNames;

	return success;
}


// --- Class ConfigParameter ---

ConfigParameter::ConfigParameter()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Type = CF_TNONE;
}

ConfigParameter::ConfigParameter(bool boolValue)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Type = CF_TBOOL;
	BoolValue = boolValue;
}

ConfigParameter::ConfigParameter(double doubleValue)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Type = CF_TDOUBLE;
	DoubleValue = doubleValue;
}

ConfigParameter::ConfigParameter(long longValue)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Type = CF_TLONG;
	LongValue = longValue;
}

ConfigParameter::ConfigParameter(const char * stringValue)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	if ((StringValue = new char[strlen(stringValue)+1]) == NULL)
	{
		Type = CF_TNONE;
		throw EOutOfMemory::Get();
	}

	Type = CF_TSTRING;
	strcpy(StringValue, stringValue);
}

ConfigParameter::~ConfigParameter()
{
	ASSERTION_THIS;

	if (Type == CF_TSTRING && StringValue != NULL)
	{
		delete StringValue;
		Type = CF_TNONE;
	}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

bool ConfigParameter::GetBoolValue()
{
	ASSERTION_THIS;

	switch (Type)
	{
		case CF_TBOOL:
			return BoolValue;

		case CF_TDOUBLE:
			return DoubleValue != 0.0;

		case CF_TLONG:
			return LongValue != 0;

		case CF_TSTRING:
			return atol(StringValue) != 0;

		default:
			return false;
	}
}

double ConfigParameter::GetDoubleValue()
{
	ASSERTION_THIS;

	switch (Type)
	{
		case CF_TBOOL:
			return BoolValue ? 1.0 : 0.0;

		case CF_TDOUBLE:
			return DoubleValue;

		case CF_TLONG:
			return (double) LongValue;

		case CF_TSTRING:
			return atof(StringValue);

		default:
			return 0.0;
	}
}

long ConfigParameter::GetLongValue()
{
	ASSERTION_THIS;

	switch (Type)
	{
		case CF_TBOOL:
			return BoolValue ? 1L : 0L;

		case CF_TDOUBLE:
			return (long) DoubleValue;

		case CF_TLONG:
			return LongValue;

		case CF_TSTRING:
			return atol(StringValue);

		default:
			return 0L;
	}
}

char * ConfigParameter::GetStringValue()
{
	ASSERTION_THIS;

	switch (Type)
	{
		case CF_TBOOL:
			return BoolValue ? "1" : "0";

		case CF_TSTRING:
			return StringValue;

		default:
			return "";
	}
}

#ifdef _TOOLBOX_TEST
	int ConfigParameter::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void ConfigParameter::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif

void ConfigParameter::SetBoolValue(bool boolValue)
{
	ASSERTION_THIS;

	if (Type == CF_TSTRING && StringValue != NULL)
	{
		delete StringValue;
		Type = CF_TNONE;
	}

	Type = CF_TBOOL;
	BoolValue = boolValue;
}

void ConfigParameter::SetDoubleValue(double doubleValue)
{
	ASSERTION_THIS;

	if (Type == CF_TSTRING && StringValue != NULL)
	{
		delete StringValue;
		Type = CF_TNONE;
	}

	Type = CF_TDOUBLE;
	DoubleValue = doubleValue;
}

void ConfigParameter::SetLongValue(long longValue)
{
	ASSERTION_THIS;

	if (Type == CF_TSTRING && StringValue != NULL)
	{
		delete StringValue;
		Type = CF_TNONE;
	}

	Type = CF_TLONG;
	LongValue = longValue;
}

void ConfigParameter::SetStringValue(const char * stringValue)
{
	ASSERTION_THIS;

	if (Type == CF_TSTRING && StringValue != NULL)
	{
		delete StringValue;
		Type = CF_TNONE;
	}

	if ((StringValue = new char[strlen(stringValue)+1]) == NULL)
	{
		Type = CF_TNONE;
		throw EOutOfMemory::Get();
	}

	Type = CF_TSTRING;
	strcpy(StringValue, stringValue);
}

bool ConfigParameter::WriteCorrectedForCfgFile(class Config::Writer * Writer)
{
	ASSERTION_THIS;

	bool success = true;

	char str[64];
	switch (Type)
	{
		case CF_TBOOL:
			success = success && Config::WriteCorrectedForCfgFile(Writer, BoolValue ? "1" : "0");
		break;

		case CF_TDOUBLE:
			sprintf(str, "%lg", DoubleValue);
			success = success && Config::WriteCorrectedForCfgFile(Writer, str);
		break;

		case CF_TLONG:
			sprintf(str, "%ld", LongValue);
			success = success && Config::WriteCorrectedForCfgFile(Writer, str);
		break;

		case CF_TSTRING:
			success = success && Config::WriteCorrectedForCfgFile(Writer, StringValue);
		break;

		default:
			success = success && Config::WriteCorrectedForCfgFile(Writer, "");
		break;
	}

	return success;
}
