/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_STACK_INL
#define __TOOLBOX_STACK_INL


// --- Class ContainingStack ---

template <class ITEMTYPE> ContainingStack<ITEMTYPE>::ContainingStack()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Top = NULL;
	Count = 0;
}

template <class ITEMTYPE> ContainingStack<ITEMTYPE>::~ContainingStack()
{
	ASSERTION_THIS;

	DeleteAll();

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

template <class ITEMTYPE> bool ContainingStack<ITEMTYPE>::Delete()
{
	ASSERTION_THIS;

	ITEMTYPE * DeleteItem;

	if ((DeleteItem = ContainingStack<ITEMTYPE>::Pop()) != NULL)
	{
		delete (ITEMTYPE *) DeleteItem;
		return true;
	}
	else
		return false;
}

template <class ITEMTYPE> void ContainingStack<ITEMTYPE>::DeleteAll()
{
	ASSERTION_THIS;

	while (Top)
		ContainingStack<ITEMTYPE>::Delete();
}

template <class ITEMTYPE> inline long ContainingStack<ITEMTYPE>::GetCount() const
{
	ASSERTION_THIS;
	return Count;
}

#ifdef _TOOLBOX_TEST
	template <class ITEMTYPE> int ContainingStack<ITEMTYPE>::InstanceCount = 0;
#endif

template <class ITEMTYPE> inline bool ContainingStack<ITEMTYPE>::IsEmpty() const
{
	ASSERTION_THIS;
	return Count<=0;
}

template <class ITEMTYPE> inline ITEMTYPE * ContainingStack<ITEMTYPE>::Peek() const
{
	ASSERTION_THIS;
	return Top;
}

template <class ITEMTYPE> ITEMTYPE * ContainingStack<ITEMTYPE>::Pop()
{
	ASSERTION_THIS;

	if (!Top)
		return NULL;

	Count--;
	ITEMTYPE * CurrentItem = Top;
	Top = Top->NextItem;

	return CurrentItem;
}

template <class ITEMTYPE> void ContainingStack<ITEMTYPE>::PopAll()
{
	ASSERTION_THIS;

	while (Top)
		ContainingStack<ITEMTYPE>::Pop();
}

template <class ITEMTYPE> void ContainingStack<ITEMTYPE>::Push(ITEMTYPE * NewItem)
{
	ASSERTION_THIS;

	if (!NewItem)
		return;

	for (ITEMTYPE * CurrentItem=NewItem; CurrentItem; CurrentItem=CurrentItem->NextItem)
		Count++;

	NewItem->NextItem = Top;
	Top = NewItem;
}

#ifdef _TOOLBOX_TEST
template <class ITEMTYPE> void ContainingStack<ITEMTYPE>::RunTestSuite(int * performedTests, int * failedTests)
{
}
#endif


// --- Class ContainingStackItem ---

template <class ITEMTYPE> ContainingStackItem<ITEMTYPE>::ContainingStackItem()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	NextItem = NULL;
}

template <class ITEMTYPE> ContainingStackItem<ITEMTYPE>::~ContainingStackItem()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

#ifdef _TOOLBOX_TEST
	template <class ITEMTYPE> int ContainingStackItem<ITEMTYPE>::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
template <class ITEMTYPE> void ContainingStackItem<ITEMTYPE>::RunTestSuite(int * performedTests, int * failedTests)
{
}
#endif


// --- Class PointeredStackItem ---

inline void * PointeredStackItem::GetData() const
{
	ASSERTION_THIS;
	return Data;
}

inline void PointeredStackItem::SetData(void * Data)
{
	ASSERTION_THIS;
	this->Data = Data;
}


#endif
